#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define NUMERO_CAMPIONI 64

//*************************************
//     Prototipi delle funzioni
//*************************************
void board_initialization (void);


int main (void){

    // Variabile per salvare la sommatoria dei dati letti
    unsigned int sommatoria = 0;

    // Variabile per salvare il valore della conversione
    unsigned int lettura = 0;

    // Variabile utilizzata per il numero delle letture
    unsigned char numero_letture;

    board_initialization ();

    // Inizializzazione LCD
    LCD_initialize (20);
    LCD_backlight (LCD_TURN_ON_LED);

    LCD_write_message ("  Temp :    ");
    LCD_write_char (223);
    LCD_write_char ('C');

    LCD_shift_cursor (LCD_LEFT,5);

    // VREF sono impostate a massa e VCC
    ADCON1 = 0b00000000;

    // Seleziono AN5 come ingresso
    ADCON0 = 0b00010100;

    // Imposto i tempi di conversione e giustificazione a destra
    // TAD : FOSC/16
    // TACQ: 16 TAD (4 min.)
    // Giustificazione destra
    ADCON2 = 0b10110101;

    // Abilito l'ADC
    ADCON0 |= 0b00000001;

    //attesa prima di iniziare la prima conversione
    //basterebbero 2us
    delay_ms (1);

    // Ciclo infinito
    while (1) {

        sommatoria = 0;

        // Effettuo NUMERO_CAMPIONI letture per fare una media
        for (numero_letture =0; numero_letture<NUMERO_CAMPIONI; numero_letture++){

            // Avvio la conversione Analogico/Digitale
            ADCON0bits.GO = 1;

            // Attendo la fine della conversione
            while(ADCON0bits.GO);

            // Prelevo il valore della conversione
            lettura = (((unsigned int) ADRESH) << 8) | ADRESL;

            // Sommatoria delle letture fatte
            sommatoria = sommatoria + lettura;
        }

        // Calcolo del valore medio
        sommatoria = sommatoria / NUMERO_CAMPIONI;

        // divido per 2 per avere la temperatura in gradi
        sommatoria = sommatoria /2;

        LCD_write_integer (sommatoria,2, LCD_ZERO_CLEANING_ON);

        LCD_shift_cursor (LCD_LEFT,2);

        delay_ms (2000);
    }
}


//*************************************
//    Implementazione della funzione
//*************************************
 void board_initialization (void) {

    // Imposto PORTA
    LATA = 0x00;    
    ANSELA = 0x00;
    TRISA = 0xFF;

    // Imposto PORTB
    LATB = 0x00;
    ANSELB = 0x00;
    TRISB = 0xFF;

    // Imposto PORTC
    LATC = 0x00;
    ANSELC = 0x00;
    TRISC = 0xFD;

    // Imposto PORTD
    LATD = 0x00;
    ANSELD = 0x00;
    TRISD = 0x00;

    // Imposto PORTE
    LATE = 0x00;
    ANSELE = 0x01;
    TRISE = 0xFF;

 }
